@@ -20,6 +20,7 @@ from .exec import Exec
 from .secret import Secret
 from .profile import Profile
 from .init import Init
+from .config import Config
 import sys
 import os
 import keeper_secrets_manager_core
@@ -30,6 +31,9 @@ import typing as t
 from update_checker import UpdateChecker
 
 
+global_config = None
+
+
 # NOTE: For the CLI, all groups and command are lowercase. All arguments are lower case, so you cannot use
 # -n and -N for an arg flag. If you add a command, you need to add it to the list of known commands so we can
 # do a best match.
@@ -124,7 +128,11 @@ class AliasedGroup(HelpColorsGroup):
         for item in args:
             # 22 is the length of the UID. We don't want to change the case of that if it starts with a -
             if item.startswith("-") and len(item) != 22:
-                item = item.lower()
+                item_parts = item.split("=")
+                item = item_parts[0].lower()
+                item_parts = item_parts[1:]
+                if len(item_parts) > 0:
+                    item += "=" + "=".join(item_parts)
             new_args.append(item)
 
         return super().parse_args(ctx, new_args)
@@ -225,8 +233,15 @@ def base_command_help(f):
 def cli(ctx, ini_file, profile_name, output, color, cache):
     """Keeper Secrets Manager CLI
     """
+
     ctx.obj = {
-        "cli": _get_cli(ini_file=ini_file, profile_name=profile_name, output=output, use_color=color, use_cache=cache),
+        "cli": _get_cli(
+            ini_file=ini_file,
+            profile_name=profile_name,
+            output=output,
+            use_color=color,
+            use_cache=cache,
+            global_config=global_config),
         "ini_file": ini_file,
         "profile_name": profile_name,
         "output": output,
@@ -295,7 +310,7 @@ def profile_list_command(ctx, json):
     if json is True:
         output = "json"
 
-    Profile(cli=ctx.obj["cli"]).list_profiles(output=output)
+    Profile(cli=ctx.obj["cli"], config=global_config).list_profiles(output=output)
 
 
 @click.command(
@@ -307,7 +322,7 @@ def profile_list_command(ctx, json):
 @click.pass_context
 def profile_active_command(ctx, profile_name):
     """Set the active profile"""
-    Profile(cli=ctx.obj["cli"]).set_active(
+    Profile(cli=ctx.obj["cli"], config=global_config).set_active(
         profile_name=profile_name
     )
 
@@ -324,7 +339,7 @@ def profile_active_command(ctx, profile_name):
 @click.pass_context
 def profile_export_command(ctx, plain, file_format, profile_name):
     """Create a new config file from a profile"""
-    Profile(cli=ctx.obj["cli"]).export_config(
+    Profile(cli=ctx.obj["cli"], config=global_config).export_config(
         plain=plain,
         file_format=file_format,
         profile_name=profile_name
@@ -341,7 +356,7 @@ def profile_export_command(ctx, plain, file_format, profile_name):
 @click.pass_context
 def profile_import_command(ctx, output_file, config_base64):
     """Import an encrypted config file"""
-    Profile(cli=ctx.obj["cli"]).import_config(
+    Profile(cli=ctx.obj["cli"], config=global_config).import_config(
         file=output_file,
         config_base64=config_base64
     )
@@ -484,16 +499,21 @@ def secret_update_command(ctx, uid, field, custom_field, field_json, custom_fiel
     help_options_color='blue'
 )
 @click.option('--uid', '-u', required=True, type=str, help="UID of the secret.")
-@click.option('--name', required=True, type=str, help='Name of the file to download.')
+@click.option('--name', type=str, help='Name of the file to download.')
+@click.option('--file-uid', type=str, help='Unique id of the file to download.')
 @click.option('--file-output', required=True, type=str, help="Where to write the file's content. "
                                                              "[filename|stdout|stderr]")
 @click.option('--create-folders', is_flag=True, help='Create folder for filename path.')
 @click.pass_context
-def secret_download_command(ctx, uid, name, file_output, create_folders):
+def secret_download_command(ctx, uid, name, file_uid, file_output, create_folders):
     """Download a file from a secret record"""
+    if name is None and file_uid is None:
+        raise KsmCliException("Either the name or file uid needs to be specified.")
+
     ctx.obj["secret"].download(
         uid=uid,
         name=name,
+        file_uid=file_uid,
         file_output=file_output,
         create_folders=create_folders
     )
@@ -769,7 +789,7 @@ def exec_command(ctx, capture_output, inline, cmd):
 @click.pass_context
 def config_command(ctx):
     """Configure the command line tool"""
-    ctx.obj["profile"] = Profile(cli=ctx.obj["cli"])
+    ctx.obj["profile"] = Profile(cli=ctx.obj["cli"], config=global_config)
     pass
 
 
@@ -867,7 +887,7 @@ config_command.add_command(config_editor_command)
 @click.pass_context
 def init_command(ctx):
     """Initialize a configuration file for integrations"""
-    ctx.obj["profile"] = Profile(cli=ctx.obj["cli"])
+    ctx.obj["profile"] = Profile(cli=ctx.obj["cli"], config=global_config)
 
 
 @click.command(
@@ -957,6 +977,9 @@ def version_command(ctx):
 def shell_command():
     """Run KSM in a shell"""
 
+    global global_config
+    global_config = Config()
+
     # https://manytools.org/hacker-tools/ascii-banner/
     logo = """
 ██╗  ██╗███████╗███╗   ███╗     ██████╗██╗     ██╗
@@ -979,7 +1002,7 @@ def shell_command():
 
     KsmCliException.in_a_shell = True
     repl(click.get_current_context(), prompt_kwargs={
-        "message": u'KSM Shell (? for help) > '
+        "message": u'\nKSM Shell (? for help) > '
     })
 
 
